@@ -1,21 +1,26 @@
 """Autocorrelation plot of data."""
-import numpy as np
-
 from ..data import convert_to_dataset
-from ..stats import autocorr
 from .plot_utils import (
     _scale_fig_size,
     default_grid,
-    make_label,
     xarray_var_iter,
-    _create_axes_grid,
     filter_plotters_list,
+    get_plotting_function,
 )
 from ..utils import _var_names
 
 
 def plot_autocorr(
-    data, var_names=None, max_lag=None, combined=False, figsize=None, textsize=None, ax=None
+    data,
+    var_names=None,
+    max_lag=None,
+    combined=False,
+    figsize=None,
+    textsize=None,
+    ax=None,
+    backend=None,
+    backend_kwargs=None,
+    show=None,
 ):
     """Bar plot of the autocorrelation function for a sequence of data.
 
@@ -40,12 +45,19 @@ def plot_autocorr(
     textsize: float
         Text size scaling factor for labels, titles and lines. If None it will be autoscaled based
         on figsize.
-    ax: axes
-        Matplotlib axes
+    ax: axes, optional
+        Matplotlib axes or bokeh figures.
+    backend: str, optional
+        Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_kwargs: bool, optional
+        These are kwargs specific to the backend being used. For additional documentation
+        check the plotting method of the backend.
+    show : bool, optional
+        Call backend show function.
 
     Returns
     -------
-    axes : matplotlib axes
+    axes : matplotlib axes or bokeh figures
 
     Examples
     --------
@@ -91,36 +103,35 @@ def plot_autocorr(
     plotters = filter_plotters_list(
         list(xarray_var_iter(data, var_names, combined)), "plot_autocorr"
     )
-    length_plotters = len(plotters)
-    rows, cols = default_grid(length_plotters)
+    rows, cols = default_grid(len(plotters))
 
     figsize, _, titlesize, xt_labelsize, linewidth, _ = _scale_fig_size(
         figsize, textsize, rows, cols
     )
 
-    if ax is None:
-        _, axes = _create_axes_grid(
-            length_plotters, rows, cols, figsize=figsize, squeeze=False, sharex=True, sharey=True
-        )
-    else:
-        axes = ax
+    autocorr_plot_args = dict(
+        axes=ax,
+        plotters=plotters,
+        max_lag=max_lag,
+        figsize=figsize,
+        rows=rows,
+        cols=cols,
+        combined=combined,
+        linewidth=linewidth,
+        xt_labelsize=xt_labelsize,
+        titlesize=titlesize,
+        backend_kwargs=backend_kwargs,
+        show=show,
+    )
 
-    axes = np.atleast_2d(axes)  # in case of only 1 plot
-    for (var_name, selection, x), ax_ in zip(plotters, axes.flatten()):
-        x_prime = x
+    if backend == "bokeh":
 
-        if combined:
-            x_prime = x.flatten()
+        autocorr_plot_args.pop("xt_labelsize")
+        autocorr_plot_args.pop("titlesize")
+        autocorr_plot_args["line_width"] = autocorr_plot_args.pop("linewidth")
 
-        y = autocorr(x_prime)
-
-        ax_.vlines(x=np.arange(0, max_lag), ymin=0, ymax=y[0:max_lag], lw=linewidth)
-        ax_.hlines(0, 0, max_lag, "steelblue")
-        ax_.set_title(make_label(var_name, selection), fontsize=titlesize, wrap=True)
-        ax_.tick_params(labelsize=xt_labelsize)
-
-    if axes.size > 0:
-        axes[0, 0].set_xlim(0, max_lag)
-        axes[0, 0].set_ylim(-1, 1)
+    # TODO: Add backend kwargs
+    plot = get_plotting_function("plot_autocorr", "autocorrplot", backend)
+    axes = plot(**autocorr_plot_args)
 
     return axes
