@@ -1,15 +1,10 @@
 """Plot a scatter or hexbin of sampled parameters."""
 import warnings
 import numpy as np
-import matplotlib.pyplot as plt
-from matplotlib.ticker import NullFormatter
-from mpl_toolkits.axes_grid1 import make_axes_locatable
 
 from ..data import convert_to_dataset, convert_to_inference_data
-from .kdeplot import plot_kde
-from .plot_utils import _scale_fig_size, xarray_to_ndarray, get_coords
+from .plot_utils import xarray_to_ndarray, get_coords, get_plotting_function
 from ..utils import _var_names
-from ..rcparams import rcParams
 
 
 def plot_pair(
@@ -27,6 +22,9 @@ def plot_pair(
     ax=None,
     divergences_kwargs=None,
     plot_kwargs=None,
+    backend=None,
+    backend_kwargs=None,
+    show=None,
 ):
     """
     Plot a scatter or hexbin matrix of the sampled parameters.
@@ -45,7 +43,7 @@ def plot_pair(
     textsize: int
         Text size for labels. If None it will be autoscaled based on figsize.
     kind : str
-        Type of plot to display (kde or hexbin)
+        Type of plot to display (scatter, kde or hexbin)
     gridsize : int or (int, int), optional
         Only works for kind=hexbin.
         The number of hexagons in the x-direction. The corresponding number of hexagons in the
@@ -61,15 +59,23 @@ def plot_pair(
     colorbar : bool
         If True a colorbar will be included as part of the plot (Defaults to False).
         Only works when kind=hexbin
-    ax: axes
-        Matplotlib axes
+    ax: axes, optional
+        Matplotlib axes or bokeh figures.
     divergences_kwargs : dicts, optional
         Additional keywords passed to ax.scatter for divergences
     plot_kwargs : dicts, optional
         Additional keywords passed to ax.plot, az.plot_kde or ax.hexbin
+    backend: str, optional
+        Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_kwargs: bool, optional
+        These are kwargs specific to the backend being used. For additional documentation
+        check the plotting method of the backend.
+    show : bool, optional
+        Call backend show function.
+
     Returns
     -------
-    ax : matplotlib axes
+    axes : matplotlib axes or bokeh figures
 
     Examples
     --------
@@ -142,6 +148,8 @@ def plot_pair(
         get_coords(posterior_data, coords), var_names=var_names, combined=True
     )
 
+    divergent_data = None
+    diverging_mask = None
     # Get diverging draws and combine chains
     if divergences:
         if hasattr(data, "sample_stats") and hasattr(data.sample_stats, "diverging"):
@@ -168,115 +176,33 @@ def plot_pair(
     if numvars < 2:
         raise Exception("Number of variables to be plotted must be 2 or greater.")
 
-    if numvars == 2:
-        (figsize, ax_labelsize, _, xt_labelsize, _, _) = _scale_fig_size(
-            figsize, textsize, numvars - 1, numvars - 1
-        )
+    pairplot_kwargs = dict(
+        ax=ax,
+        _posterior=_posterior,
+        numvars=numvars,
+        figsize=figsize,
+        textsize=textsize,
+        kind=kind,
+        plot_kwargs=plot_kwargs,
+        contour=contour,
+        fill_last=fill_last,
+        gridsize=gridsize,
+        colorbar=colorbar,
+        divergences=divergences,
+        diverging_mask=diverging_mask,
+        divergences_kwargs=divergences_kwargs,
+        flat_var_names=flat_var_names,
+        backend_kwargs=backend_kwargs,
+        show=show,
+    )
 
-        if ax is None:
-            fig, ax = plt.subplots(figsize=figsize, constrained_layout=True)
-
-        if kind == "scatter":
-            ax.plot(_posterior[0], _posterior[1], **plot_kwargs)
-        elif kind == "kde":
-            plot_kde(
-                _posterior[0],
-                _posterior[1],
-                contour=contour,
-                fill_last=fill_last,
-                ax=ax,
-                **plot_kwargs
-            )
-        else:
-            hexbin = ax.hexbin(
-                _posterior[0], _posterior[1], mincnt=1, gridsize=gridsize, **plot_kwargs
-            )
-            ax.grid(False)
-
-        if kind == "hexbin" and colorbar:
-            cbar = ax.figure.colorbar(hexbin, ticks=[hexbin.norm.vmin, hexbin.norm.vmax], ax=ax)
-            cbar.ax.set_yticklabels(["low", "high"], fontsize=ax_labelsize)
-
-        if divergences:
-            ax.plot(
-                _posterior[0][diverging_mask], _posterior[1][diverging_mask], **divergences_kwargs
-            )
-
-        ax.set_xlabel("{}".format(flat_var_names[0]), fontsize=ax_labelsize, wrap=True)
-        ax.set_ylabel("{}".format(flat_var_names[1]), fontsize=ax_labelsize, wrap=True)
-        ax.tick_params(labelsize=xt_labelsize)
-
-    else:
-        max_plots = (
-            numvars ** 2 if rcParams["plot.max_subplots"] is None else rcParams["plot.max_subplots"]
-        )
-        vars_to_plot = np.sum(np.arange(numvars).cumsum() < max_plots)
-        if vars_to_plot < numvars:
-            warnings.warn(
-                "rcParams['plot.max_subplots'] ({max_plots}) is smaller than the number "
-                "of resulting pair plots with these variables, generating only a "
-                "{side}x{side} grid".format(max_plots=max_plots, side=vars_to_plot),
-                SyntaxWarning,
-            )
-            numvars = vars_to_plot
-
-        (figsize, ax_labelsize, _, xt_labelsize, _, _) = _scale_fig_size(
-            figsize, textsize, numvars - 2, numvars - 2
-        )
-
-        if ax is None:
-            fig, ax = plt.subplots(
-                numvars - 1, numvars - 1, figsize=figsize, constrained_layout=True
-            )
-        hexbin_values = []
-        for i in range(0, numvars - 1):
-            var1 = _posterior[i]
-
-            for j in range(0, numvars - 1):
-                if j < i:
-                    ax[j, i].axis("off")
-                    continue
-
-                var2 = _posterior[j + 1]
-
-                if kind == "scatter":
-                    ax[j, i].plot(var1, var2, **plot_kwargs)
-
-                elif kind == "kde":
-                    plot_kde(
-                        var1, var2, contour=contour, fill_last=fill_last, ax=ax[j, i], **plot_kwargs
-                    )
-
-                else:
-                    ax[j, i].grid(False)
-                    hexbin = ax[j, i].hexbin(var1, var2, mincnt=1, gridsize=gridsize, **plot_kwargs)
-                if kind == "hexbin" and colorbar:
-                    hexbin_values.append(hexbin.norm.vmin)
-                    hexbin_values.append(hexbin.norm.vmax)
-                    if j == i == 0 and colorbar:
-                        divider = make_axes_locatable(ax[0, 1])
-                        cax = divider.append_axes("left", size="7%")
-                        cbar = fig.colorbar(
-                            hexbin, ticks=[hexbin.norm.vmin, hexbin.norm.vmax], cax=cax
-                        )
-                        cbar.ax.set_yticklabels(["low", "high"], fontsize=ax_labelsize)
-
-                if divergences:
-                    ax[j, i].plot(var1[diverging_mask], var2[diverging_mask], **divergences_kwargs)
-
-                if j + 1 != numvars - 1:
-                    ax[j, i].axes.get_xaxis().set_major_formatter(NullFormatter())
-                else:
-                    ax[j, i].set_xlabel(
-                        "{}".format(flat_var_names[i]), fontsize=ax_labelsize, wrap=True
-                    )
-                if i != 0:
-                    ax[j, i].axes.get_yaxis().set_major_formatter(NullFormatter())
-                else:
-                    ax[j, i].set_ylabel(
-                        "{}".format(flat_var_names[j + 1]), fontsize=ax_labelsize, wrap=True
-                    )
-
-                ax[j, i].tick_params(labelsize=xt_labelsize)
+    if backend == "bokeh":
+        pairplot_kwargs.pop("gridsize", None)
+        pairplot_kwargs.pop("colorbar", None)
+        pairplot_kwargs.pop("divergences_kwargs", None)
+        pairplot_kwargs.pop("hexbin_values", None)
 
+    # TODO: Add backend kwargs
+    plot = get_plotting_function("plot_pair", "pairplot", backend)
+    ax = plot(**pairplot_kwargs)
     return ax
