@@ -1,10 +1,9 @@
 """Parallel coordinates plot showing posterior points with and without divergences marked."""
-import matplotlib.pyplot as plt
 import numpy as np
-
 from scipy.stats.mstats import rankdata
+
 from ..data import convert_to_dataset
-from .plot_utils import _scale_fig_size, xarray_to_ndarray, get_coords
+from .plot_utils import _scale_fig_size, xarray_to_ndarray, get_coords, get_plotting_function
 from ..utils import _var_names, _numba_var
 from ..stats.stats_utils import stats_variance_2d as svar
 
@@ -21,6 +20,9 @@ def plot_parallel(
     shadend=0.025,
     ax=None,
     norm_method=None,
+    backend=None,
+    backend_kwargs=None,
+    show=None,
 ):
     """
     Plot parallel coordinates plot showing posterior points with and without divergences.
@@ -51,15 +53,22 @@ def plot_parallel(
     shadend : float
         Alpha blending value for non-divergent points, between 0 (invisible) and 1 (opaque).
         Defaults to .025
-    ax : axes
-        Matplotlib axes.
+    ax: axes, optional
+        Matplotlib axes or bokeh figures.
     norm_method : str
         Method for normalizing the data. Methods include normal, minmax and rank.
         Defaults to none.
+    backend: str, optional
+        Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_kwargs: bool, optional
+        These are kwargs specific to the backend being used. For additional documentation
+        check the plotting method of the backend.
+    show : bool, optional
+        Call backend show function.
 
     Returns
     -------
-    ax : matplotlib axes
+    axes : matplotlib axes or bokeh figures
 
     Examples
     --------
@@ -118,22 +127,33 @@ def plot_parallel(
 
     figsize, _, _, xt_labelsize, _, _ = _scale_fig_size(figsize, textsize, 1, 1)
 
-    if ax is None:
-        _, ax = plt.subplots(figsize=figsize, constrained_layout=True)
+    parallel_kwargs = dict(
+        ax=ax,
+        colornd=colornd,
+        colord=colord,
+        shadend=shadend,
+        diverging_mask=diverging_mask,
+        _posterior=_posterior,
+        textsize=textsize,
+        var_names=var_names,
+        xt_labelsize=xt_labelsize,
+        legend=legend,
+        figsize=figsize,
+        backend_kwargs=backend_kwargs,
+        show=show,
+    )
 
-    ax.plot(_posterior[:, ~diverging_mask], color=colornd, alpha=shadend)
+    if backend == "bokeh":
 
-    if np.any(diverging_mask):
-        ax.plot(_posterior[:, diverging_mask], color=colord, lw=1)
+        parallel_kwargs.pop("textsize")
+        parallel_kwargs.pop("xt_labelsize")
+        parallel_kwargs.pop("legend")
+        parallel_kwargs.pop("colord")
+        parallel_kwargs.pop("colornd")
+        parallel_kwargs.pop("shadend")
 
-    ax.tick_params(labelsize=textsize)
-    ax.set_xticks(range(len(var_names)))
-    ax.set_xticklabels(var_names)
-
-    if legend:
-        ax.plot([], color=colornd, label="non-divergent")
-        if np.any(diverging_mask):
-            ax.plot([], color=colord, label="divergent")
-        ax.legend(fontsize=xt_labelsize)
+    # TODO: Add backend kwargs
+    plot = get_plotting_function("plot_parallel", "parallelplot", backend)
+    ax = plot(**parallel_kwargs)
 
     return ax
