@@ -1,12 +1,15 @@
 """Utilities for plotting."""
 import warnings
 from itertools import product, tee
+import importlib
 
+import packaging
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib as mpl
 import xarray as xr
 
+
 from ..utils import conditional_jit
 from ..rcparams import rcParams
 
@@ -130,6 +133,28 @@ def get_bins(values):
     return np.arange(x_min, x_max + width + 1, width)
 
 
+def _sturges_formula(dataset, mult=1):
+    """Use Sturges' formula to determine number of bins.
+
+    See https://en.wikipedia.org/wiki/Histogram#Sturges'_formula
+    or https://doi.org/10.1080%2F01621459.1926.10502161
+
+    Parameters
+    ----------
+    dataset: xarray.DataSet
+        Must have the `draw` dimension
+
+    mult: float
+        Used to scale the number of bins up or down. Default is 1 for Sturges' formula.
+
+    Returns
+    -------
+    int
+        Number of bins to use
+    """
+    return int(np.ceil(mult * np.log2(dataset.draw.size)) + 1)
+
+
 def default_grid(n_items, max_cols=4, min_cols=3):  # noqa: D202
     """Make a grid for subplots.
 
@@ -166,7 +191,7 @@ def default_grid(n_items, max_cols=4, min_cols=3):  # noqa: D202
     return n_items // ideal + 1, ideal
 
 
-def _create_axes_grid(length_plotters, rows, cols, **kwargs):
+def _create_axes_grid(length_plotters, rows, cols, backend=None, backend_kwargs=None, **kwargs):
     """Create figure and axes for grids with multiple plots.
 
     Parameters
@@ -177,20 +202,73 @@ def _create_axes_grid(length_plotters, rows, cols, **kwargs):
         Number of rows
     cols : int
         Number of columns
+    backend: str, optional
+        Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
+    backend_kwargs: dict, optional
+        kwargs for backend figure.
 
     Returns
     -------
     fig : matplotlib figure
     ax : matplotlib axes
     """
-    kwargs.setdefault("constrained_layout", True)
-    fig, ax = plt.subplots(rows, cols, **kwargs)
-    ax = np.ravel(ax)
-    extra = (rows * cols) - length_plotters
-    if extra:
-        for i in range(1, extra + 1):
-            ax[-i].set_axis_off()
-        ax = ax[:-extra]
+    if backend_kwargs is None:
+        backend_kwargs = {}
+
+    if backend == "bokeh":
+        from bokeh.plotting import figure
+        from .backends.bokeh import backend_kwarg_defaults
+
+        backend_kwargs = {
+            **backend_kwarg_defaults(
+                ("tools", "plot.bokeh.tools"),
+                ("output_backend", "plot.bokeh.output_backend"),
+                ("width", "plot.bokeh.figure.width"),
+                ("height", "plot.bokeh.figure.height"),
+                ("dpi", "plot.bokeh.figure.dpi"),
+            ),
+            **backend_kwargs,
+        }
+        dpi = backend_kwargs.pop("dpi")
+        if "figsize" in kwargs:
+            figsize = kwargs["figsize"]
+            figsize = int(figsize[0] * dpi / cols), int(figsize[1] * dpi / rows)
+            backend_kwargs["width"] = figsize[0]
+            backend_kwargs["height"] = figsize[1]
+
+        sharex = kwargs.get("sharex", False)
+        sharey = kwargs.get("sharey", False)
+        fig = None
+        ax = []
+        extra = (rows * cols) - length_plotters
+        for row in range(rows):
+            row_ax = []
+            for col in range(cols):
+                if (row == 0) and (col == 0) and (sharex or sharey):
+                    bokeh_ax = figure(**backend_kwargs)
+                    row_ax.append(bokeh_ax)
+                    if sharex:
+                        backend_kwargs["x_range"] = bokeh_ax.x_range
+                    if sharey:
+                        backend_kwargs["y_range"] = bokeh_ax.y_range
+                else:
+                    if row * cols + (col + 1) > length_plotters:
+                        row_ax.append(None)
+                    else:
+                        row_ax.append(figure(**backend_kwargs))
+            ax.append(row_ax)
+        ax = np.array(ax)
+    else:
+        from .backends.matplotlib import backend_kwarg_defaults
+
+        backend_kwargs = {**backend_kwarg_defaults(), **backend_kwargs, **kwargs}
+        fig, ax = plt.subplots(rows, cols, **backend_kwargs)
+        ax = np.ravel(ax)
+        extra = (rows * cols) - length_plotters
+        if extra:
+            for i in range(1, extra + 1):
+                ax[-i].set_axis_off()
+            ax = ax[:-extra]
     return fig, ax
 
 
@@ -238,6 +316,40 @@ def make_label(var_name, selection, position="below"):
     return "{}{}{}".format(var_name, sep, sel)
 
 
+def format_sig_figs(value, default=None):
+    """Get a default number of significant figures.
+
+    Gives the integer part or `default`, whichever is bigger.
+
+    Examples
+    --------
+    0.1234 --> 0.12
+    1.234  --> 1.2
+    12.34  --> 12
+    123.4  --> 123
+    """
+    if default is None:
+        default = 2
+    if value == 0:
+        return 1
+    return max(int(np.log10(np.abs(value))) + 1, default)
+
+
+def round_num(n, round_to):
+    """
+    Return a string representing a number with `round_to` significant figures.
+
+    Parameters
+    ----------
+    n : float
+        number to round
+    round_to : int
+        number of significant figures
+    """
+    sig_figs = format_sig_figs(n, round_to)
+    return "{n:.{sig_figs}g}".format(n=n, sig_figs=sig_figs)
+
+
 @conditional_jit(forceobj=True)
 def purge_duplicates(list_in):
     """Remove duplicates from list while preserving order.
@@ -422,21 +534,33 @@ def get_coords(data, coords):
     data: xarray
         xarray.DataSet or xarray.DataArray object, same type as input
     """
-    try:
-        return data.sel(**coords)
+    if not isinstance(data, (list, tuple)):
+        try:
+            return data.sel(**coords)
 
-    except ValueError:
-        invalid_coords = set(coords.keys()) - set(data.coords.keys())
-        raise ValueError("Coords {} are invalid coordinate keys".format(invalid_coords))
+        except ValueError:
+            invalid_coords = set(coords.keys()) - set(data.coords.keys())
+            raise ValueError("Coords {} are invalid coordinate keys".format(invalid_coords))
 
-    except KeyError as err:
-        raise KeyError(
-            (
-                "Coords should follow mapping format {{coord_name:[dim1, dim2]}}. "
-                "Check that coords structure is correct and"
-                " dimensions are valid. {}"
-            ).format(err)
-        )
+        except KeyError as err:
+            raise KeyError(
+                (
+                    "Coords should follow mapping format {{coord_name:[dim1, dim2]}}. "
+                    "Check that coords structure is correct and"
+                    " dimensions are valid. {}"
+                ).format(err)
+            )
+    if not isinstance(coords, (list, tuple)):
+        coords = [coords] * len(data)
+    data_subset = []
+    for idx, (datum, coords_dict) in enumerate(zip(data, coords)):
+        try:
+            data_subset.append(get_coords(datum, coords_dict))
+        except ValueError as err:
+            raise ValueError("Error in data[{}]: {}".format(idx, err))
+        except KeyError as err:
+            raise KeyError("Error in data[{}]: {}".format(idx, err))
+    return data_subset
 
 
 def color_from_dim(dataarray, dim_name):
@@ -507,3 +631,45 @@ def filter_plotters_list(plotters, plot_kind):
         )
         return plotters[:max_plots]
     return plotters
+
+
+def get_plotting_function(plot_name, plot_module, backend):
+    """Return plotting function for correct backend."""
+    _backend = {
+        "mpl": "matplotlib",
+        "bokeh": "bokeh",
+        "matplotlib": "matplotlib",
+        None: "matplotlib",
+    }
+
+    try:
+        backend = _backend[backend]
+    except KeyError:
+        raise KeyError(
+            "Backend {} is not implemented. Try backend in {}".format(
+                backend, set(_backend.values())
+            )
+        )
+
+    if backend == "bokeh":
+        try:
+            import bokeh
+
+            assert packaging.version.parse(bokeh.__version__) >= packaging.version.parse("1.4.0")
+
+        except (ImportError, AssertionError):
+            raise ImportError(
+                "'bokeh' backend needs Bokeh (1.4.0+) installed." " Please upgrade or install"
+            )
+
+    # Perform import of plotting method
+    # TODO: Convert module import to top level for all plots
+    module = importlib.import_module(
+        "arviz.plots.backends.{backend}.{plot_module}".format(
+            backend=backend, plot_module=plot_module
+        )
+    )
+
+    plotting_method = getattr(module, plot_name)
+
+    return plotting_method
