@@ -6,20 +6,45 @@ import re
 import pprint
 import logging
 import locale
-from collections import MutableMapping
+from collections.abc import MutableMapping
 
 _log = logging.getLogger(__name__)
 
 
-def _make_validate_choice(accepted_values):
-    """Validate value is in accepted_values."""
+def _make_validate_choice(accepted_values, allow_none=False, typeof=str):
+    """Validate value is in accepted_values.
+
+    Parameters
+    ----------
+    accepted_values : iterable
+        Iterable containing all accepted_values.
+    allow_none: boolean, optional
+        Whether to accept ``None`` in addition to the values in ``accepted_values``.
+    typeof: type, optional
+        Type the values should be converted to.
+    """
     # no blank lines allowed after function docstring by pydocstyle,
     # but black requires white line before function
 
     def validate_choice(value):
-        if value.lower() in accepted_values:
-            return value.lower()
-        raise ValueError("{} is not one of {}".format(value, accepted_values))
+        if allow_none and (value is None or isinstance(value, str) and value.lower() == "none"):
+            return None
+        try:
+            value = typeof(value)
+        except (ValueError, TypeError):
+            raise ValueError("Could not convert to {}".format(typeof.__name__))
+        if isinstance(value, str):
+            value = value.lower()
+
+        if value in accepted_values:
+            # Convert value to python boolean if string matches
+            value = {"true": True, "false": False}.get(value, value)
+            return value
+        raise ValueError(
+            "{} is not one of {}{}".format(
+                value, accepted_values, " nor None" if allow_none else ""
+            )
+        )
 
     return validate_choice
 
@@ -44,10 +69,54 @@ def _validate_positive_int_or_none(value):
         return _validate_positive_int(value)
 
 
+def _validate_float(value):
+    """Validate value is a float."""
+    try:
+        value = float(value)
+    except ValueError:
+        raise ValueError("Could not convert to float")
+    return value
+
+
+def _validate_probability(value):
+    """Validate a probability: a float between 0 and 1."""
+    value = _validate_float(value)
+    if (value < 0) or (value > 1):
+        raise ValueError("Only values between 0 and 1 are valid.")
+    return value
+
+
+def _validate_boolean(value):
+    """Validate value is a float."""
+    if value not in {True, "true", False, "false"}:
+        raise ValueError("Only boolean values are valid.")
+    return value is True or value == "true"
+
+
 defaultParams = {  # pylint: disable=invalid-name
-    "data.load": ("lazy", _make_validate_choice(("lazy", "eager"))),
+    "data.http_protocol": ("https", _make_validate_choice({"https", "http"})),
+    "data.load": ("lazy", _make_validate_choice({"lazy", "eager"})),
+    "data.index_origin": (0, _make_validate_choice({0, 1}, typeof=int)),
+    "plot.backend": ("matplotlib", _make_validate_choice({"matplotlib", "bokeh"})),
+    "plot.bokeh.tools": (
+        "pan,box_zoom,wheel_zoom,box_select,lasso_select,undo,redo,reset,save,hover",
+        lambda x: x,
+    ),
+    "plot.bokeh.output_backend": ("webgl", _make_validate_choice({"canvas", "svg", "webgl"})),
+    "plot.bokeh.figure.dpi": (60, _validate_positive_int),
+    "plot.bokeh.figure.width": (500, _validate_positive_int),
+    "plot.bokeh.figure.height": (500, _validate_positive_int),
+    "plot.bokeh.show": (True, _validate_boolean),
+    "plot.matplotlib.constrained_layout": (True, _validate_boolean),
+    "plot.matplotlib.show": (False, _validate_boolean),
     "plot.max_subplots": (40, _validate_positive_int_or_none),
-    "stats.information_criterion": ("waic", _make_validate_choice(("waic", "loo"))),
+    "plot.point_estimate": (
+        "mean",
+        _make_validate_choice({"mean", "median", "mode"}, allow_none=True),
+    ),
+    "stats.credible_interval": (0.94, _validate_probability),
+    "stats.information_criterion": ("waic", _make_validate_choice({"waic", "loo"})),
+    "stats.ic_scale": ("deviance", _make_validate_choice({"deviance", "log", "negative_log"})),
 }
 
 
@@ -85,6 +154,42 @@ class RcParams(MutableMapping, dict):  # pylint: disable=too-many-ancestors
         """Raise TypeError if someone ever tries to delete a key from RcParams."""
         raise TypeError("RcParams keys cannot be deleted")
 
+    def clear(self):
+        """Raise TypeError if someone ever tries to delete all keys from RcParams."""
+        raise TypeError("RcParams keys cannot be deleted")
+
+    def pop(self, key, default=None):
+        """Raise TypeError if someone ever tries to delete a key from RcParams."""
+        raise TypeError(
+            "RcParams keys cannot be deleted. Use .get(key) of RcParams[key] to check values"
+        )
+
+    def popitem(self):
+        """Raise TypeError if someone ever tries to delete a key from RcParams."""
+        raise TypeError(
+            "RcParams keys cannot be deleted. Use .get(key) of RcParams[key] to check values"
+        )
+
+    def setdefault(self, key, default=None):
+        """Raise error when using setdefault, defaults are handled on initialization."""
+        raise TypeError(
+            "Defaults in RcParams are handled on object initialization during library"
+            "import. Use arvizrc file instead."
+            ""
+        )
+
+    def items(self):
+        """Explicit use of MutableMapping attributes."""
+        return MutableMapping.items(self)
+
+    def keys(self):
+        """Explicit use of MutableMapping attributes."""
+        return MutableMapping.keys(self)
+
+    def values(self):
+        """Explicit use of MutableMapping attributes."""
+        return MutableMapping.values(self)
+
     def __repr__(self):
         """Customize repr of RcParams objects."""
         class_name = self.__class__.__name__
