@@ -485,9 +485,9 @@ def ks_summary(pareto_tail_indices):
     _numba_flag = Numba.numba_flag
     if _numba_flag:
         bins = np.asarray([-np.Inf, 0.5, 0.7, 1, np.Inf])
-        kcounts, _ = histogram(pareto_tail_indices, bins)
+        kcounts, *_ = histogram(pareto_tail_indices, bins)
     else:
-        kcounts, _ = np.histogram(pareto_tail_indices, bins=[-np.Inf, 0.5, 0.7, 1, np.Inf])
+        kcounts, *_ = histogram(pareto_tail_indices, bins=[-np.Inf, 0.5, 0.7, 1, np.Inf])
     kprop = kcounts / len(pareto_tail_indices) * 100
     df_k = pd.DataFrame(
         dict(_=["(good)", "(ok)", "(bad)", "(very bad)"], Count=kcounts, Pct=kprop)
@@ -751,7 +751,7 @@ def _ess_quantile(ary, prob, relative=False):
         return np.nan
     if prob is None:
         raise TypeError("Prob not defined.")
-    quantile, = _quantile(ary, prob)
+    (quantile,) = _quantile(ary, prob)
     iquantile = ary <= quantile
     return _ess(_split_chains(iquantile), relative=relative)
 
