@@ -4,6 +4,7 @@ import warnings
 import logging
 from collections import OrderedDict
 from copy import deepcopy
+from typing import Optional, List, Union
 
 import numpy as np
 import pandas as pd
@@ -11,7 +12,9 @@ import scipy.stats as st
 from scipy.optimize import minimize
 import xarray as xr
 
-from ..data import convert_to_inference_data, convert_to_dataset, InferenceData
+from ..data import convert_to_inference_data, convert_to_dataset, InferenceData, CoordSpec, DimSpec
+from ..plots.kdeplot import _fast_kde
+from ..plots.plot_utils import get_bins
 from .diagnostics import _multichain_statistics, _mc_error, ess, _circular_standard_deviation
 from .stats_utils import (
     make_ufunc as _make_ufunc,
@@ -21,6 +24,7 @@ from .stats_utils import (
     stats_variance_2d as svar,
 )
 from ..utils import _var_names, Numba, _numba_var
+from ..stats.stats_utils import histogram
 from ..rcparams import rcParams
 
 _log = logging.getLogger(__name__)
@@ -305,27 +309,30 @@ def _ic_matrix(ics, ic_i):
     return rows, cols, ic_i_val
 
 
-def hpd(ary, credible_interval=0.94, circular=False):
+def hpd(ary, credible_interval=0.94, circular=False, multimodal=False):
     """
     Calculate highest posterior density (HPD) of array for given credible_interval.
 
-    The HPD is the minimum width Bayesian credible interval (BCI). This implementation works only
-    for unimodal distributions.
+    The HPD is the minimum width Bayesian credible interval (BCI).
 
     Parameters
     ----------
-    x : Numpy array
+    ary : Numpy array
         An array containing posterior samples
     credible_interval : float, optional
         Credible interval to compute. Defaults to 0.94.
     circular : bool, optional
         Whether to compute the hpd taking into account `x` is a circular variable
         (in the range [-np.pi, np.pi]) or not. Defaults to False (i.e non-circular variables).
+        Only works if multimodal is False.
+    multimodal : bool
+        If true it may compute more than one hpd interval if the distribution is multimodal and the
+        modes are well separated.
 
     Returns
     -------
     np.ndarray
-        lower and upper value of the interval.
+        lower(s) and upper(s) values of the interval(s).
 
     Examples
     --------
@@ -340,40 +347,79 @@ def hpd(ary, credible_interval=0.94, circular=False):
     """
     if ary.ndim > 1:
         hpd_array = np.array(
-            [hpd(row, credible_interval=credible_interval, circular=circular) for row in ary.T]
+            [
+                hpd(
+                    row,
+                    credible_interval=credible_interval,
+                    circular=circular,
+                    multimodal=multimodal,
+                )
+                for row in ary.T
+            ]
         )
         return hpd_array
-    # Make a copy of trace
-    ary = ary.copy()
-    n = len(ary)
 
-    if circular:
-        mean = st.circmean(ary, high=np.pi, low=-np.pi)
-        ary = ary - mean
-        ary = np.arctan2(np.sin(ary), np.cos(ary))
+    if multimodal:
+        if ary.dtype.kind == "f":
+            density, lower, upper = _fast_kde(ary)
+            range_x = upper - lower
+            dx = range_x / len(density)
+            bins = np.linspace(lower, upper, len(density))
+        else:
+            bins = get_bins(ary)
+            _, density, _ = histogram(ary, bins=bins)
+            dx = np.diff(bins)[0]
 
-    ary = np.sort(ary)
-    interval_idx_inc = int(np.floor(credible_interval * n))
-    n_intervals = n - interval_idx_inc
-    interval_width = ary[interval_idx_inc:] - ary[:n_intervals]
+        density *= dx
 
-    if len(interval_width) == 0:
-        raise ValueError(
-            "Too few elements for interval calculation. "
-            "Check that credible_interval meets condition 0 =< credible_interval < 1"
-        )
+        idx = np.argsort(-density)
+        intervals = bins[idx][density[idx].cumsum() <= credible_interval]
+        intervals.sort()
 
-    min_idx = np.argmin(interval_width)
-    hdi_min = ary[min_idx]
-    hdi_max = ary[min_idx + interval_idx_inc]
+        intervals_splitted = np.split(intervals, np.where(np.diff(intervals) >= dx * 1.1)[0] + 1)
 
-    if circular:
-        hdi_min = hdi_min + mean
-        hdi_max = hdi_max + mean
-        hdi_min = np.arctan2(np.sin(hdi_min), np.cos(hdi_min))
-        hdi_max = np.arctan2(np.sin(hdi_max), np.cos(hdi_max))
+        hpd_intervals = []
+        for interval in intervals_splitted:
+            if interval.size == 0:
+                hpd_intervals.append((bins[0], bins[0]))
+            else:
+                hpd_intervals.append((interval[0], interval[-1]))
 
-    return np.array([hdi_min, hdi_max])
+        hpd_intervals = np.array(hpd_intervals)
+
+    else:
+        ary = ary.copy()
+        n = len(ary)
+
+        if circular:
+            mean = st.circmean(ary, high=np.pi, low=-np.pi)
+            ary = ary - mean
+            ary = np.arctan2(np.sin(ary), np.cos(ary))
+
+        ary = np.sort(ary)
+        interval_idx_inc = int(np.floor(credible_interval * n))
+        n_intervals = n - interval_idx_inc
+        interval_width = ary[interval_idx_inc:] - ary[:n_intervals]
+
+        if len(interval_width) == 0:
+            raise ValueError(
+                "Too few elements for interval calculation. "
+                "Check that credible_interval meets condition 0 =< credible_interval < 1"
+            )
+
+        min_idx = np.argmin(interval_width)
+        hdi_min = ary[min_idx]
+        hdi_max = ary[min_idx + interval_idx_inc]
+
+        if circular:
+            hdi_min = hdi_min + mean
+            hdi_max = hdi_max + mean
+            hdi_min = np.arctan2(np.sin(hdi_min), np.cos(hdi_min))
+            hdi_max = np.arctan2(np.sin(hdi_max), np.cos(hdi_max))
+
+        hpd_intervals = np.array([hdi_min, hdi_max])
+
+    return hpd_intervals
 
 
 def loo(data, pointwise=False, reff=None, scale="deviance"):
@@ -439,14 +485,10 @@ def loo(data, pointwise=False, reff=None, scale="deviance"):
 
     """
     inference_data = convert_to_inference_data(data)
-    for group in ("posterior", "sample_stats"):
-        if not hasattr(inference_data, group):
-            raise TypeError(
-                "Must be able to extract a {group} group from data!".format(group=group)
-            )
+    if not hasattr(inference_data, "sample_stats"):
+        raise TypeError("Must be able to extract a sample_stats group from data.")
     if "log_likelihood" not in inference_data.sample_stats:
         raise TypeError("Data must include log_likelihood in sample_stats")
-    posterior = inference_data.posterior
     log_likelihood = inference_data.sample_stats.log_likelihood
     log_likelihood = log_likelihood.stack(sample=("chain", "draw"))
     shape = log_likelihood.shape
@@ -463,6 +505,9 @@ def loo(data, pointwise=False, reff=None, scale="deviance"):
         raise TypeError('Valid scale values are "deviance", "log", "negative_log"')
 
     if reff is None:
+        if not hasattr(inference_data, "posterior"):
+            raise TypeError("Must be able to extract a posterior group from data.")
+        posterior = inference_data.posterior
         n_chains = len(posterior.chain)
         if n_chains == 1:
             reff = 1.0
@@ -619,7 +664,7 @@ def _psislw(log_weights, cutoff_ind, cutoffmin, k_min=1.0 / 3):
     xcutoff = max(x[x_sort_ind[cutoff_ind]], cutoffmin)
 
     expxcutoff = np.exp(xcutoff)
-    tailinds, = np.where(x > xcutoff)  # pylint: disable=unbalanced-tuple-unpacking
+    (tailinds,) = np.where(x > xcutoff)  # pylint: disable=unbalanced-tuple-unpacking
     x_tail = x[tailinds]
     tail_len = len(x_tail)
     if tail_len <= 4:
@@ -757,8 +802,9 @@ def r2_score(y_true, y_pred):
 
 def summary(
     data,
-    var_names=None,
-    fmt="wide",
+    var_names: Optional[List[str]] = None,
+    fmt: str = "wide",
+    kind: str = "all",
     round_to=None,
     include_circ=None,
     stat_funcs=None,
@@ -766,7 +812,9 @@ def summary(
     credible_interval=0.94,
     order="C",
     index_origin=0,
-):
+    coords: Optional[CoordSpec] = None,
+    dims: Optional[DimSpec] = None,
+) -> Union[pd.DataFrame, xr.Dataset]:
     """Create a data frame with summary statistics.
 
     Parameters
@@ -776,12 +824,16 @@ def summary(
         Refer to documentation of az.convert_to_dataset for details
     var_names : list
         Names of variables to include in summary
-    include_circ : bool
-        Whether to include circular statistics
     fmt : {'wide', 'long', 'xarray'}
         Return format is either pandas.DataFrame {'wide', 'long'} or xarray.Dataset {'xarray'}.
+    kind : {'all', 'stats', 'diagnostics'}
+        Whether to include the `stats`: `mean`, `sd`, `hpd_3%`, `hpd_97%`, or the `diagnostics`:
+        `mcse_mean`, `mcse_sd`, `ess_bulk`, `ess_tail`, and `r_hat`. Default to include `all` of
+        them.
     round_to : int
         Number of decimals used to round results. Defaults to 2. Use "none" to return raw numbers.
+    include_circ : bool
+        Whether to include circular statistics
     stat_funcs : dict
         A list of functions or a dict of functions with function names as keys used to calculate
         statistics. By default, the mean, standard deviation, simulation standard error, and
@@ -800,12 +852,18 @@ def summary(
         If fmt is "wide", use either C or F unpacking order. Defaults to C.
     index_origin : int
         If fmt is "wide, select n-based indexing for multivariate parameters. Defaults to 0.
+    coords: Dict[str, List[Any]], optional
+        Coordinates specification to be used if the ``fmt`` is ``'xarray'``.
+    dims: Dict[str, List[str]], optional
+        Dimensions specification for the variables to be used if the ``fmt`` is ``'xarray'``.
 
     Returns
     -------
-    pandas.DataFrame
-        With summary statistics for each variable. Defaults statistics are: `mean`, `sd`,
-        `hpd_3%`, `hpd_97%`, `mcse_mean`, `mcse_sd`, `ess_bulk`, `ess_tail` and `r_hat`.
+    pandas.DataFrame or xarray.Dataset
+        Return type dicated by `fmt` argument.
+        Return value will contain summary statistics for each variable. Default statistics are:
+        `mean`, `sd`, `hpd_3%`, `hpd_97%`, `mcse_mean`, `mcse_sd`, `ess_bulk`, `ess_tail`, and
+        `r_hat`.
         `r_hat` is only computed for traces with 2 or more chains.
 
     Examples
@@ -842,18 +900,23 @@ def summary(
            ...: )
 
     """
-    posterior = convert_to_dataset(data, group="posterior")
+    extra_args = {}  # type: Dict[str, Any]
+    if coords is not None:
+        extra_args["coords"] = coords
+    if dims is not None:
+        extra_args["dims"] = dims
+    posterior = convert_to_dataset(data, group="posterior", **extra_args)
     var_names = _var_names(var_names, posterior)
     posterior = posterior if var_names is None else posterior[var_names]
 
     fmt_group = ("wide", "long", "xarray")
     if not isinstance(fmt, str) or (fmt.lower() not in fmt_group):
-        raise TypeError("Invalid format: '{}'! Formatting options are: {}".format(fmt, fmt_group))
+        raise TypeError("Invalid format: '{}'. Formatting options are: {}".format(fmt, fmt_group))
 
     unpack_order_group = ("C", "F")
     if not isinstance(order, str) or (order.upper() not in unpack_order_group):
         raise TypeError(
-            "Invalid order: '{}'! Unpacking options are: {}".format(order, unpack_order_group)
+            "Invalid order: '{}'. Unpacking options are: {}".format(order, unpack_order_group)
         )
 
     alpha = 1 - credible_interval
@@ -879,7 +942,7 @@ def summary(
                 )
                 extra_metric_names.append(stat_func.__name__)
 
-    if extend:
+    if extend and kind in ["all", "stats"]:
         mean = posterior.mean(dim=("chain", "draw"))
 
         sd = posterior.std(dim=("chain", "draw"), ddof=1)
@@ -887,7 +950,7 @@ def summary(
         hpd_lower, hpd_higher = xr.apply_ufunc(
             _make_ufunc(hpd, n_output=2),
             posterior,
-            kwargs=dict(credible_interval=credible_interval),
+            kwargs=dict(credible_interval=credible_interval, multimodal=False),
             input_core_dims=(("chain", "draw"),),
             output_core_dims=tuple([] for _ in range(2)),
         )
@@ -927,19 +990,33 @@ def summary(
             output_core_dims=tuple([] for _ in range(2)),
         )
 
-    mcse_mean, mcse_sd, ess_mean, ess_sd, ess_bulk, ess_tail, r_hat = xr.apply_ufunc(
-        _make_ufunc(_multichain_statistics, n_output=7, ravel=False),
-        posterior,
-        input_core_dims=(("chain", "draw"),),
-        output_core_dims=tuple([] for _ in range(7)),
-    )
+    if kind in ["all", "diagnostics"]:
+        mcse_mean, mcse_sd, ess_mean, ess_sd, ess_bulk, ess_tail, r_hat = xr.apply_ufunc(
+            _make_ufunc(_multichain_statistics, n_output=7, ravel=False),
+            posterior,
+            input_core_dims=(("chain", "draw"),),
+            output_core_dims=tuple([] for _ in range(7)),
+        )
 
     # Combine metrics
     metrics = []
     metric_names = []
     if extend:
-        metrics.extend(
-            (
+        metrics_names_ = (
+            "mean",
+            "sd",
+            "hpd_{:g}%".format(100 * alpha / 2),
+            "hpd_{:g}%".format(100 * (1 - alpha / 2)),
+            "mcse_mean",
+            "mcse_sd",
+            "ess_mean",
+            "ess_sd",
+            "ess_bulk",
+            "ess_tail",
+            "r_hat",
+        )
+        if kind == "all":
+            metrics_ = (
                 mean,
                 sd,
                 hpd_lower,
@@ -952,22 +1029,14 @@ def summary(
                 ess_tail,
                 r_hat,
             )
-        )
-        metric_names.extend(
-            (
-                "mean",
-                "sd",
-                "hpd_{:g}%".format(100 * alpha / 2),
-                "hpd_{:g}%".format(100 * (1 - alpha / 2)),
-                "mcse_mean",
-                "mcse_sd",
-                "ess_mean",
-                "ess_sd",
-                "ess_bulk",
-                "ess_tail",
-                "r_hat",
-            )
-        )
+        elif kind == "stats":
+            metrics_ = (mean, sd, hpd_lower, hpd_higher)
+            metrics_names_ = metrics_names_[:4]
+        elif kind == "diagnostics":
+            metrics_ = (mcse_mean, mcse_sd, ess_mean, ess_sd, ess_bulk, ess_tail, r_hat)
+            metrics_names_ = metrics_names_[4:]
+        metrics.extend(metrics_)
+        metric_names.extend(metrics_names_)
     if include_circ:
         metrics.extend((circ_mean, circ_sd, circ_hpd_lower, circ_hpd_higher, circ_mcse))
         metric_names.extend(
@@ -1011,6 +1080,7 @@ def summary(
         df.index = list(df.index)
         summary_df = df
     else:
+        # format is 'xarray'
         summary_df = joined
     if (round_to is not None) and (round_to not in ("None", "none")):
         summary_df = summary_df.round(round_to)
@@ -1088,7 +1158,7 @@ def waic(data, pointwise=False, scale="deviance"):
     for group in ("sample_stats",):
         if not hasattr(inference_data, group):
             raise TypeError(
-                "Must be able to extract a {group} group from data!".format(group=group)
+                "Must be able to extract a {group} group from data.".format(group=group)
             )
     if "log_likelihood" not in inference_data.sample_stats:
         raise TypeError("Data must include log_likelihood in sample_stats")
