@@ -1,4 +1,4 @@
-# pylint: disable=redefined-outer-name
+# pylint: disable=redefined-outer-name, comparison-with-callable
 """Test helper functions."""
 import gzip
 import importlib
@@ -217,7 +217,7 @@ def _emcee_lnprior(theta):
     if tau < 0:
         return -np.inf
     prior_tau = -np.log(tau ** 2 + 25 ** 2)
-    prior_mu = -(mu / 10) ** 2  # normal prior, loc=0, scale=10
+    prior_mu = -((mu / 10) ** 2)  # normal prior, loc=0, scale=10
     prior_eta = -np.sum(eta ** 2)  # normal prior, loc=0, scale=1
     return prior_mu + prior_tau + prior_eta
 
@@ -226,7 +226,7 @@ def _emcee_lnprob(theta, y, sigma):
     """Proper function to allow pickling."""
     mu, tau, eta = theta[0], theta[1], theta[2:]
     prior = _emcee_lnprior(theta)
-    like_vect = -((mu + tau * eta - y) / sigma) ** 2
+    like_vect = -(((mu + tau * eta - y) / sigma) ** 2)
     like = np.sum(like_vect)
     return like + prior, (like_vect, np.random.normal((mu + tau * eta), sigma))
 
@@ -242,7 +242,7 @@ def emcee_schools_model(data, draws, chains):
     ndim = J + 2
 
     pos = np.random.normal(size=(chains, ndim))
-    pos[:, 1] = np.absolute(pos[:, 1])
+    pos[:, 1] = np.absolute(pos[:, 1])  #  pylint: disable=unsupported-assignment-operation
 
     if emcee_version() < 3:
         sampler = emcee.EnsembleSampler(chains, ndim, _emcee_lnprob, args=(y, sigma))
@@ -263,81 +263,69 @@ def emcee_schools_model(data, draws, chains):
     return sampler
 
 
-# pylint:disable=no-member,no-value-for-parameter
-def _pyro_centered_model(sigma):
-    """Centered model setup."""
+# pylint:disable=no-member,no-value-for-parameter,invalid-name
+def _pyro_noncentered_model(J, sigma, y=None):
     import pyro
-    import torch
     import pyro.distributions as dist
 
-    mu = pyro.sample("mu", dist.Normal(torch.zeros(1), 10 * torch.ones(1)))
-    tau = pyro.sample("tau", dist.HalfCauchy(scale=25 * torch.ones(1)))
-
-    theta = pyro.sample("theta", dist.Normal(mu * torch.ones(8), tau * torch.ones(8)))
-
-    return pyro.sample("obs", dist.Normal(theta, sigma))
+    mu = pyro.sample("mu", dist.Normal(0, 5))
+    tau = pyro.sample("tau", dist.HalfCauchy(5))
+    with pyro.plate("J", J):
+        eta = pyro.sample("eta", dist.Normal(0, 1))
+        theta = mu + tau * eta
+        return pyro.sample("obs", dist.Normal(theta, sigma), obs=y)
 
 
-def _pyro_conditioned_model(model, sigma, y):
-    """Condition the model."""
-    import pyro
-
-    return pyro.poutine.condition(model, data={"obs": y})(sigma)
-
-
-def pyro_centered_schools(data, draws, chains):
-    """Centered eight schools implementation in Pyro.
-
-    Note there is not really a deterministic node in pyro, so I do not
-    know how to do a non-centered implementation.
-    """
+def pyro_noncentered_schools(data, draws, chains):
+    """Non-centered eight schools implementation in Pyro."""
     import torch
-    from pyro.infer.mcmc import MCMC, NUTS
+    from pyro.infer import MCMC, NUTS
 
-    del chains
-    y = torch.Tensor(data["y"]).type(torch.Tensor)
-    sigma = torch.Tensor(data["sigma"]).type(torch.Tensor)
+    y = torch.from_numpy(data["y"]).float()
+    sigma = torch.from_numpy(data["sigma"]).float()
 
-    nuts_kernel = NUTS(_pyro_conditioned_model, adapt_step_size=True)
-    posterior = MCMC(  # pylint:disable=not-callable
-        nuts_kernel, num_samples=draws, warmup_steps=500
-    ).run(_pyro_centered_model, sigma, y)
+    nuts_kernel = NUTS(_pyro_noncentered_model, jit_compile=True, ignore_jit_warnings=True)
+    posterior = MCMC(nuts_kernel, num_samples=draws, warmup_steps=draws, num_chains=chains)
+    posterior.run(data["J"], sigma, y)
 
     # This block lets the posterior be pickled
-    for trace in posterior.exec_traces:
-        for node in trace.nodes.values():
-            node.pop("fn", None)
-    posterior.kernel = None
-    posterior.run = None
-    posterior.logger = None
-    if hasattr(posterior, "sampler"):
-        posterior.sampler = None
+    posterior.sampler = None
+    posterior.kernel.potential_fn = None
     return posterior
 
 
+# pylint:disable=no-member,no-value-for-parameter,invalid-name
+def _numpyro_noncentered_model(J, sigma, y=None):
+    import numpyro
+    import numpyro.distributions as dist
+
+    mu = numpyro.sample("mu", dist.Normal(0, 5))
+    tau = numpyro.sample("tau", dist.HalfCauchy(5))
+    with numpyro.plate("J", J):
+        eta = numpyro.sample("eta", dist.Normal(0, 1))
+        theta = mu + tau * eta
+        return numpyro.sample("obs", dist.Normal(theta, sigma), obs=y)
+
+
 def numpyro_schools_model(data, draws, chains):
     """Centered eight schools implementation in NumPyro."""
-    import jax
-    import numpyro
-    import numpyro.distributions as dist
-    from numpyro.mcmc import MCMC, NUTS
-
-    def model():
-        mu = numpyro.sample("mu", dist.Normal(0, 5))
-        tau = numpyro.sample("tau", dist.HalfCauchy(5))
-        # TODO: use numpyro.plate or `sample_shape` kwargs instead of  # pylint: disable=fixme
-        # multiplying with np.ones(J) in future versions of NumPyro
-        theta = numpyro.sample("theta", dist.Normal(mu * np.ones(data["J"]), tau))
-        numpyro.sample("obs", dist.Normal(theta, data["sigma"]), obs=data["y"])
+    from jax.random import PRNGKey
+    from numpyro.infer import MCMC, NUTS
 
     mcmc = MCMC(
-        NUTS(model),
+        NUTS(_numpyro_noncentered_model),
         num_warmup=draws,
         num_samples=draws,
         num_chains=chains,
         chain_method="sequential",
     )
-    mcmc.run(jax.random.PRNGKey(0), collect_fields=("z", "diverging"))
+    mcmc.run(PRNGKey(0), extra_fields=("num_steps", "energy"), **data)
+
+    # This block lets the posterior be pickled
+    mcmc.sampler._sample_fn = None  # pylint: disable=protected-access
+    mcmc.sampler._init_fn = None  # pylint: disable=protected-access
+    mcmc.sampler._constrain_fn = None  # pylint: disable=protected-access
+    mcmc._cache = {}  # pylint: disable=protected-access
     return mcmc
 
 
@@ -346,6 +334,11 @@ def tfp_schools_model(num_schools, treatment_stddevs):
     import tensorflow_probability.python.edward2 as ed
     import tensorflow as tf
 
+    if int(tf.__version__[0]) > 1:
+        import tensorflow.compat.v1 as tf  # pylint: disable=import-error
+
+        tf.disable_v2_behavior()
+
     avg_effect = ed.Normal(loc=0.0, scale=10.0, name="avg_effect")  # `mu`
     avg_stddev = ed.Normal(loc=5.0, scale=1.0, name="avg_stddev")  # `log(tau)`
     school_effects_standard = ed.Normal(
@@ -364,6 +357,11 @@ def tfp_noncentered_schools(data, draws, chains):
     import tensorflow_probability.python.edward2 as ed
     import tensorflow as tf
 
+    if int(tf.__version__[0]) > 1:
+        import tensorflow.compat.v1 as tf  # pylint: disable=import-error
+
+        tf.disable_v2_behavior()
+
     del chains
 
     log_joint = ed.make_log_joint_fn(tfp_schools_model)
@@ -440,7 +438,12 @@ def pystan_noncentered_schools(data, draws, chains):
 
         stan_model = pystan.StanModel(model_code=schools_code)
         fit = stan_model.sampling(
-            data=data, iter=draws, warmup=0, chains=chains, check_hmc_diagnostics=False
+            data=data,
+            iter=draws,
+            warmup=0,
+            chains=chains,
+            check_hmc_diagnostics=False,
+            control=dict(adapt_engaged=False),
         )
     else:
         import stan  # pylint: disable=import-error
@@ -486,7 +489,7 @@ def load_cached_models(eight_schools_data, draws, chains, libs=None):
         ("pystan", pystan_noncentered_schools),
         ("pymc3", pymc3_noncentered_schools),
         ("emcee", emcee_schools_model),
-        ("pyro", pyro_centered_schools),
+        ("pyro", pyro_noncentered_schools),
         ("numpyro", numpyro_schools_model),
     )
     data_directory = os.path.join(here, "saved_models")
@@ -505,11 +508,6 @@ def load_cached_models(eight_schools_data, draws, chains, libs=None):
             _log.info("Generating and loading stan model")
             models["pystan"] = func(eight_schools_data, draws, chains)
             continue
-        elif library.__name__ == "numpyro":
-            # NumPyro does not support pickling
-            _log.info("Generating and loading NumPyro model")
-            models["numpyro"] = func(eight_schools_data, draws, chains)
-            continue
 
         py_version = sys.version_info
         fname = "{0.major}.{0.minor}_{1.__name__}_{1.__version__}_{2}_{3}_{4}.pkl.gzip".format(
