@@ -2,10 +2,13 @@
 # pylint: disable=too-many-lines
 from collections import namedtuple
 import os
+from typing import Dict
 from urllib.parse import urlunsplit
 import numpy as np
 import pytest
 
+import xarray as xr
+
 from arviz import (
     concat,
     convert_to_inference_data,
@@ -73,7 +76,19 @@ def no_remote_data(monkeypatch, tmpdir):
 
 
 def test_load_local_arviz_data():
-    assert load_arviz_data("centered_eight")
+    inference_data = load_arviz_data("centered_eight")
+    assert isinstance(inference_data, InferenceData)
+    assert set(inference_data.observed_data.obs.coords["school"].values) == {
+        "Hotchkiss",
+        "Mt. Hermon",
+        "Choate",
+        "Deerfield",
+        "Phillips Andover",
+        "St. Paul's",
+        "Lawrenceville",
+        "Phillips Exeter",
+    }
+    assert inference_data.posterior["theta"].dims == ("chain", "draw", "school")
 
 
 def test_clear_data_home():
@@ -288,6 +303,22 @@ def test_concat_bad():
         concat(idata3, idata, dim="chain")
 
 
+def test_inference_concat_keeps_all_fields():
+    """From failures observed in issue #907"""
+    idata1 = from_dict(posterior={"A": [1, 2, 3, 4]}, sample_stats={"B": [2, 3, 4, 5]})
+    idata2 = from_dict(prior={"C": [1, 2, 3, 4]}, observed_data={"D": [2, 3, 4, 5]})
+
+    idata_c1 = concat(idata1, idata2)
+    idata_c2 = concat(idata2, idata1)
+
+    test_dict = {"posterior": ["A"], "sample_stats": ["B"], "prior": ["C"], "observed_data": ["D"]}
+
+    fails_c1 = check_multiple_attrs(test_dict, idata_c1)
+    assert not fails_c1
+    fails_c2 = check_multiple_attrs(test_dict, idata_c2)
+    assert not fails_c2
+
+
 @pytest.mark.parametrize("inplace", [True, False])
 def test_sel_method(inplace):
     data = np.random.normal(size=(4, 500, 8))
@@ -318,6 +349,25 @@ def test_sel_method(inplace):
             assert np.all(dataset.draw.values == np.arange(200, ndraws))
 
 
+def test_sel_method_chain_prior():
+    idata = load_arviz_data("centered_eight")
+    original_groups = getattr(idata, "_groups")
+    idata_subset = idata.sel(inplace=False, chain_prior=False, chain=[0, 1, 3])
+    groups = getattr(idata_subset, "_groups")
+    assert np.all(np.isin(groups, original_groups))
+    for group in groups:
+        dataset_subset = getattr(idata_subset, group)
+        dataset = getattr(idata, group)
+        if "chain" in dataset.dims:
+            assert "chain" in dataset_subset.dims
+            if "prior" not in group:
+                assert np.all(dataset_subset.chain.values == np.array([0, 1, 3]))
+        else:
+            assert "chain" not in dataset_subset.dims
+    with pytest.raises(KeyError):
+        idata.sel(inplace=False, chain_prior=True, chain=[0, 1, 3])
+
+
 @pytest.mark.parametrize("use", ("del", "delattr"))
 def test_del_method(use):
     # create inference data object
@@ -753,3 +803,63 @@ class TestDataNetCDF:
         assert os.path.exists(filepath)
         os.remove(filepath)
         assert not os.path.exists(filepath)
+
+
+class TestConversions:
+    def test_id_conversion_idempotent(self):
+        stored = load_arviz_data("centered_eight")
+        inference_data = convert_to_inference_data(stored)
+        assert isinstance(inference_data, InferenceData)
+        assert set(inference_data.observed_data.obs.coords["school"].values) == {
+            "Hotchkiss",
+            "Mt. Hermon",
+            "Choate",
+            "Deerfield",
+            "Phillips Andover",
+            "St. Paul's",
+            "Lawrenceville",
+            "Phillips Exeter",
+        }
+        assert inference_data.posterior["theta"].dims == ("chain", "draw", "school")
+
+    def test_dataset_conversion_idempotent(self):
+        inference_data = load_arviz_data("centered_eight")
+        data_set = convert_to_dataset(inference_data.posterior)
+        assert isinstance(data_set, xr.Dataset)
+        assert set(data_set.coords["school"].values) == {
+            "Hotchkiss",
+            "Mt. Hermon",
+            "Choate",
+            "Deerfield",
+            "Phillips Andover",
+            "St. Paul's",
+            "Lawrenceville",
+            "Phillips Exeter",
+        }
+        assert data_set["theta"].dims == ("chain", "draw", "school")
+
+    def test_id_conversion_args(self):
+        stored = load_arviz_data("centered_eight")
+        IVIES = ["Yale", "Harvard", "MIT", "Princeton", "Cornell", "Dartmouth", "Columbia", "Brown"]
+        # test dictionary argument...
+        # I reverse engineered a dictionary out of the centered_eight
+        # data. That's what this block of code does.
+        d = stored.posterior.to_dict()
+        d = d["data_vars"]
+        test_dict = {}  # type: Dict[str, np.ndarray]
+        for var_name in d:
+            data = d[var_name]["data"]
+            # this is a list of chains that is a list of samples...
+            chain_arrs = []
+            for chain in data:  # list of samples
+                chain_arrs.append(np.array(chain))
+            data_arr = np.stack(chain_arrs)
+            test_dict[var_name] = data_arr
+
+        inference_data = convert_to_inference_data(
+            test_dict, dims={"theta": ["Ivies"]}, coords={"Ivies": IVIES}
+        )
+
+        assert isinstance(inference_data, InferenceData)
+        assert set(inference_data.posterior.coords["Ivies"].values) == set(IVIES)
+        assert inference_data.posterior["theta"].dims == ("chain", "draw", "Ivies")
