@@ -1,7 +1,7 @@
 # pylint: disable=redefined-outer-name, no-member
 from copy import deepcopy
 import numpy as np
-from numpy.testing import assert_allclose, assert_array_almost_equal
+from numpy.testing import assert_allclose, assert_array_almost_equal, assert_array_equal
 import pytest
 from scipy.stats import linregress
 from xarray import Dataset, DataArray
@@ -47,6 +47,20 @@ def test_hpd():
     assert_array_almost_equal(interval, [-1.88, 1.88], 2)
 
 
+def test_hpd_multimodal():
+    normal_sample = np.concatenate(
+        (np.random.normal(-4, 1, 2500000), np.random.normal(2, 0.5, 2500000))
+    )
+    intervals = hpd(normal_sample, multimodal=True)
+    assert_array_almost_equal(intervals, [[-5.8, -2.2], [0.9, 3.1]], 1)
+
+
+def test_hpd_circular():
+    normal_sample = np.random.vonmises(np.pi, 1, 5000000)
+    interval = hpd(normal_sample, circular=True)
+    assert_array_almost_equal(interval, [0.6, -0.6], 1)
+
+
 def test_hpd_bad_ci():
     normal_sample = np.random.randn(10)
     with pytest.raises(ValueError):
@@ -142,6 +156,31 @@ def test_summary_include_circ(centered_eight, include_circ):
     assert state == Numba.numba_flag
 
 
+METRICS_NAMES = [
+    "mean",
+    "sd",
+    "hpd_3%",
+    "hpd_97%",
+    "mcse_mean",
+    "mcse_sd",
+    "ess_mean",
+    "ess_sd",
+    "ess_bulk",
+    "ess_tail",
+    "r_hat",
+]
+
+
+@pytest.mark.parametrize(
+    "params",
+    (("all", METRICS_NAMES), ("stats", METRICS_NAMES[:4]), ("diagnostics", METRICS_NAMES[4:])),
+)
+def test_summary_kind(centered_eight, params):
+    kind, metrics_names_ = params
+    summary_df = summary(centered_eight, kind=kind)
+    assert_array_equal(summary_df.columns, metrics_names_)
+
+
 @pytest.mark.parametrize("fmt", ["wide", "long", "xarray"])
 def test_summary_fmt(centered_eight, fmt):
     assert summary(centered_eight, fmt=fmt) is not None
@@ -307,6 +346,15 @@ def test_loo_bad_scale(centered_eight):
         loo(centered_eight, scale="bad_scale")
 
 
+def test_loo_bad_no_posterior_reff(centered_eight):
+    loo(centered_eight, reff=None)
+    centered_eight = deepcopy(centered_eight)
+    del centered_eight.posterior
+    with pytest.raises(TypeError):
+        loo(centered_eight, reff=None)
+    loo(centered_eight, reff=0.7)
+
+
 def test_loo_warning(centered_eight):
     centered_eight = deepcopy(centered_eight)
     # make one of the khats infinity
